'''
    This script takes a raw xml file as input and outputs
    an almost clean csv file
'''

import re, csv
from bs4 import BeautifulSoup

# file paths
IN = 'raw xml file'
OUT = 'almost clean csv file'

# regex patterns
TAG_PATTERN = re.compile('<|>')
NEW_LINE = re.compile('\n')
BODY_1 = re.compile('\n?<pre(.*?)><code>(.*?)<\/code><\/pre>\n?|<div class=".*<div class=".*>', flags=re.S)
BODY_2 = re.compile('\s?<code>\w*<\/code>.\s?|\s?<a href=.*\">|<\/a>')
BODY_3 = re.compile('\s?<\w*>|<\/\w*>(.|\s)?')
BODY_4 = re.compile('&nbsp;|http(s?)://\S*|<img src.*">|<br/>*|br>|<br />*|<ol.*">|/p>|/strong>|/li>|&lt;key>|&lt;/key>|/code>|/a>|#?&\w\w\w?;?')

# method to clean instances using regex
def cleanup(dirty):
    clean = re.sub(NEW_LINE,' ', dirty)    
    clean = re.sub(BODY_1,' ', clean)
    clean = re.sub(BODY_2,' ', clean)
    clean = re.sub(BODY_3,' ', clean)
    clean = re.sub(BODY_4,' ', clean)
    return clean


open(OUT, 'w').close() # erase file berfore writing
with open(IN, 'r') as inFile, open(OUT, 'a') as outFile: 
    outFileWriter = csv.writer(outFile, delimiter=';')
    outFileWriter.writerow(['id', 'post', 'title', 'tags']) # write header
    for post in inFile:
        soup = BeautifulSoup(post, "lxml") # get xml tree
        line = soup.find("row")
        newRow = [] # new list to append as a row to the output file
        try:
            typeId = line.attrs['posttypeid']
            newRow.append(line.attrs['id']) # first field 

            if typeId == '1':
                newRow.append(cleanup(line.attrs['body'])) # second field
                newRow.append(line.attrs['title']) # third field
                newRow.append(re.sub(TAG_PATTERN,' ', line.attrs['tags'])) # fourth field

            elif typeId == '2': # title and tags attributes are missing      
                newRow.append(cleanup(line.attrs['body'])) # second field 
                # leave title and tag fields empty
                newRow.append('') # third field
                newRow.append('') # fourth field
            
            outFileWriter.writerow(newRow) # append cleaned row tothe file
            
        except:
            pass
    