import csv
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.svm import LinearSVC

## create model ##

TRAIN = 'training set (csv)'

# lists used to hold posts and labels
trainLabels = []
trainPosts = []

# populate lists
with open(TRAIN) as file:
    fileReader = csv.reader(file, delimiter=';')
    next(fileReader) # skip header row
    for row in fileReader:
        trainLabels.append(row[0])
        trainPosts.append(row[1])

# get features from the file obtained after the feature selection phase done on weka
attributes = [line.rstrip('\n') for line in open('features file (txt)')]
# use those as vocabulary for the vectorization
vect = CountVectorizer(vocabulary=set(attributes), ngram_range=(1, 4))
# vectorize posts
trainMatrix = vect.fit_transform(trainPosts)
# model
model = LinearSVC(C=0.25, max_iter=5000)
model.fit(trainMatrix, trainLabels)

print('\nModel trained\n')


## Predict labels ##

IN = ['first dataset labelled by the first classifier', 'second dataset labelled by the first classifier', '...']
PRED = ['first dataset labelled by the second classifier', 'second dataset labelled by the second classifier', '...']

for inFile, outFile in zip(IN, PRED):
    print('Labelling ', inFile, '\t.....', end='\r')
    # lists used to keep the data of each instance
    inFileId = []
    inFileTags = []
    inFileDate = []
    inFileTitle = []
    inFilePosts = []
    # populate lists
    with open(inFile) as file:
        fileReader = csv.reader(file, delimiter=';')
        next(fileReader) # skip header row
        for row in fileReader:
            if row[0] == 'subjective':
                inFileId.append(row[3])
                inFileTags.append(row[4])
                inFileDate.append(row[5])
                inFileTitle.append(row[6])
                inFilePosts.append(row[7])

    # vectorize posts
    predMatrix = vect.transform(inFilePosts)
    # predict labels
    predLabel = model.predict(predMatrix)
    predProb = model._predict_proba_lr(predMatrix)
 

    ## Create new csv with the predicted labels

    open(outFile, mode='w').close()
    with open(outFile, mode='a') as out:
        writer = csv.writer(out, delimiter=';')
        writer.writerow(['label', 'positive', 'negative', 'id', 'tags', 'date', 'title', 'posts'])
        i = 1
        for i in range(0, len(inFilePosts)):
            newArray = []
            newArray.append(predLabel[i])
            newArray.append(predProb[i][0])
            newArray.append(predProb[i][1])
            newArray.append(inFileId[i])
            newArray.append(inFileTags[i])
            newArray.append(inFileDate[i])
            newArray.append(inFileTitle[i])
            newArray.append(inFilePosts[i])
            writer.writerow(newArray)

    print(outFile, '......................DONE', '\n')

print('DONE LABELLING\n')
